package net.sourceforge.nattable.filterrow.config;

import static net.sourceforge.nattable.config.CellConfigAttributes.CELL_PAINTER;
import static net.sourceforge.nattable.config.IEditableRule.ALWAYS_EDITABLE;
import static net.sourceforge.nattable.edit.EditConfigAttributes.CELL_EDITABLE_RULE;
import static net.sourceforge.nattable.filterrow.TextMatchingMode.CONTAINS;
import static net.sourceforge.nattable.filterrow.config.FilterRowConfigAttributes.FILTER_COMPARATOR;
import static net.sourceforge.nattable.filterrow.config.FilterRowConfigAttributes.FILTER_DISPLAY_CONVERTER;
import static net.sourceforge.nattable.filterrow.config.FilterRowConfigAttributes.TEXT_MATCHING_MODE;
import static net.sourceforge.nattable.grid.GridRegion.FILTER_ROW;
import static net.sourceforge.nattable.style.DisplayMode.NORMAL;
import net.sourceforge.nattable.config.AbstractRegistryConfiguration;
import net.sourceforge.nattable.config.DefaultComparator;
import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.data.convert.DefaultDisplayConverter;
import net.sourceforge.nattable.edit.action.MouseEditAction;
import net.sourceforge.nattable.filterrow.FilterRowPainter;
import net.sourceforge.nattable.filterrow.TextMatchingMode;
import net.sourceforge.nattable.filterrow.action.ClearFilterAction;
import net.sourceforge.nattable.filterrow.action.ToggleFilterRowAction;
import net.sourceforge.nattable.filterrow.event.ClearFilterIconMouseEventMatcher;
import net.sourceforge.nattable.filterrow.event.FilterRowMouseEventMatcher;
import net.sourceforge.nattable.painter.cell.ICellPainter;
import net.sourceforge.nattable.ui.binding.UiBindingRegistry;
import net.sourceforge.nattable.ui.matcher.KeyEventMatcher;

import org.eclipse.swt.SWT;

public class DefaultFilterRowConfiguration extends AbstractRegistryConfiguration {

	public ICellPainter cellPainter = new FilterRowPainter();
	public TextMatchingMode textMatchingMode = CONTAINS;
	public int showHideKeyConstant = SWT.F3;

	public void configureRegistry(IConfigRegistry configRegistry) {
		// Plug in custom painter
		configRegistry.registerConfigAttribute(CELL_PAINTER, cellPainter, NORMAL, FILTER_ROW);

		// Make cells editable
		configRegistry.registerConfigAttribute(CELL_EDITABLE_RULE, ALWAYS_EDITABLE, NORMAL, FILTER_ROW);

		// Default text matching mode
		configRegistry.registerConfigAttribute(TEXT_MATCHING_MODE, textMatchingMode);

		// Default display converter. Used to convert the values typed into the text boxes into String objects.
		configRegistry.registerConfigAttribute(FILTER_DISPLAY_CONVERTER, new DefaultDisplayConverter());

		// Default comparator. Used to compare objects in the column during threshold matching.
		configRegistry.registerConfigAttribute(FILTER_COMPARATOR, DefaultComparator.getInstance());
	}

	@Override
	public void configureUiBindings(UiBindingRegistry uiBindingRegistry) {
		uiBindingRegistry.registerFirstSingleClickBinding(new FilterRowMouseEventMatcher(), new MouseEditAction());
		uiBindingRegistry.registerFirstSingleClickBinding(new ClearFilterIconMouseEventMatcher(), new ClearFilterAction());
		uiBindingRegistry.registerKeyBinding(new KeyEventMatcher(showHideKeyConstant), new ToggleFilterRowAction());
	}
}