package net.sourceforge.nattable.group;

import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.painter.cell.CellPainterWrapper;
import net.sourceforge.nattable.painter.cell.ICellPainter;
import net.sourceforge.nattable.painter.cell.ImagePainter;
import net.sourceforge.nattable.painter.cell.TextPainter;
import net.sourceforge.nattable.painter.cell.decorator.CellPainterDecorator;
import net.sourceforge.nattable.ui.util.CellEdgeEnum;
import net.sourceforge.nattable.util.GUIHelper;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;

public class RowGroupHeaderTextPainter extends CellPainterWrapper {

	/** Needed to query row group cell expand/collapse state */
	private final RowGroupModel rowGroupModel;
	
	public RowGroupHeaderTextPainter(RowGroupModel rowGroupModel) {
		this.rowGroupModel = rowGroupModel;
		
		setWrappedPainter(new CellPainterDecorator(new TextPainter(), CellEdgeEnum.BOTTOM, new ExpandCollapseImagePainter()));
	}
	
	/**
	 * @param rowGroupModel Column group model used by the grid
	 * @param interiorPainter for painting the text portion
	 * @param imagePainter for painting the icon image on the right
	 */
	public RowGroupHeaderTextPainter(RowGroupModel rowGroupModel, ICellPainter interiorPainter) {
		this.rowGroupModel = rowGroupModel;
		
		setWrappedPainter(new CellPainterDecorator(interiorPainter, CellEdgeEnum.BOTTOM, new ExpandCollapseImagePainter()));
	}
	
	@Override
	public int getPreferredWidth(LayerCell cell, GC gc, IConfigRegistry configRegistry) {
		return 0;
	}

	public class ExpandCollapseImagePainter extends ImagePainter {
		final Image downImg = GUIHelper.getImage("down_0"); //$NON-NLS-1$
		final Image upImg = GUIHelper.getImage("up_0"); //$NON-NLS-1$

		public ExpandCollapseImagePainter() {
			super(null, false);
		}

		@Override
		protected Image getImage(LayerCell cell, IConfigRegistry configRegistry) {
			String cellValue = cell.getDataValue().toString();
			if (rowGroupModel.isAGroup(cellValue)) {
				return rowGroupModel.isCollapsed(cellValue) ? downImg : upImg;
			} else {
				return null;
			}
		}

	}
}
