package net.sourceforge.nattable.group.command;

import java.util.ArrayList;
import java.util.List;

import net.sourceforge.nattable.command.AbstractLayerCommandHandler;
import net.sourceforge.nattable.group.RowGroupExpandCollapseLayer;
import net.sourceforge.nattable.group.RowGroupModel;
import net.sourceforge.nattable.group.RowGroupModel.RowGroup;
import net.sourceforge.nattable.hideshow.event.HideRowPositionsEvent;
import net.sourceforge.nattable.hideshow.event.ShowRowPositionsEvent;
import net.sourceforge.nattable.layer.event.ILayerEvent;

public class RowGroupExpandCollapseCommandHandler extends AbstractLayerCommandHandler<RowGroupExpandCollapseCommand> {

	private final RowGroupExpandCollapseLayer rowGroupExpandCollapseLayer;

	public RowGroupExpandCollapseCommandHandler(RowGroupExpandCollapseLayer rowGroupExpandCollapseLayer) {
		this.rowGroupExpandCollapseLayer = rowGroupExpandCollapseLayer;
	}
	
	public Class<RowGroupExpandCollapseCommand> getCommandClass() {
		return RowGroupExpandCollapseCommand.class;
	}

	@Override
	protected boolean doCommand(RowGroupExpandCollapseCommand command) {
		
		int rowIndex = rowGroupExpandCollapseLayer.getRowIndexByPosition(command.getRowPosition());
		RowGroupModel model = rowGroupExpandCollapseLayer.getModel();
		
		// if group of columnIndex is not collapseable return without any 
		// further operation ...
		if (!model.isCollapseable(rowIndex)) {
			return true;
		}
		
		boolean wasCollapsed = model.isCollapsed(rowIndex);
		RowGroup rowGroup = model.toggleRowGroupExpandCollapse(rowIndex);
		
		List<Integer> rowIndexes = new ArrayList<Integer>(rowGroup.getMembers());
		rowIndexes.removeAll(rowGroup.getStaticRowIndexes());
		List<Integer> rowPositions = new ArrayList<Integer>();
		for (Integer memberRowIndex : rowIndexes) {
		    rowPositions.add(rowGroupExpandCollapseLayer.getRowPositionByIndex(memberRowIndex));
		}
		
		ILayerEvent event;
		if (wasCollapsed) {
			event = new ShowRowPositionsEvent(rowGroupExpandCollapseLayer, rowPositions);
		} else {
			event = new HideRowPositionsEvent(rowGroupExpandCollapseLayer, rowPositions);
		}
		
		rowGroupExpandCollapseLayer.fireLayerEvent(event);
		
		return true;
	}
	
}
