package net.sourceforge.nattable.selection;

import static net.sourceforge.nattable.selection.SelectionUtils.isControlOnly;
import static net.sourceforge.nattable.selection.SelectionUtils.noShiftOrControl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sourceforge.nattable.command.AbstractLayerCommandHandler;
import net.sourceforge.nattable.coordinate.Range;
import net.sourceforge.nattable.group.RowGroupExpandCollapseLayer;
import net.sourceforge.nattable.group.RowGroupModel;
import net.sourceforge.nattable.selection.command.SelectRowGroupsCommand;
import net.sourceforge.nattable.selection.event.RowSelectionEvent;

import org.eclipse.swt.graphics.Rectangle;

public class SelectRowGroupCommandHandler extends AbstractLayerCommandHandler<SelectRowGroupsCommand> {

	private final RowGroupExpandCollapseLayer rowGroupHeaderLayer;
	private final SelectionLayer selectionLayer;

	public SelectRowGroupCommandHandler(RowGroupExpandCollapseLayer rowGroupHeaderLayer, SelectionLayer selectionLayer) {
		this.rowGroupHeaderLayer = rowGroupHeaderLayer;
		this.selectionLayer = selectionLayer;
	}
	
	public Class<SelectRowGroupsCommand> getCommandClass() {
		return SelectRowGroupsCommand.class;
	}

	@Override
	protected boolean doCommand(SelectRowGroupsCommand command) {		
		final RowGroupModel model = rowGroupHeaderLayer.getModel();
		final List<Integer> rowIndexes = model.getRowIndexesInGroup( rowGroupHeaderLayer.getRowIndexByPosition( command.getRowPosition() ) );
		final List<Integer> rowPositions = model.getRowPositionsInGroup( rowGroupHeaderLayer, rowIndexes );		
		selectRows(command.getColumnPosition(), rowPositions, command.isWithShiftMask(), command.isWithControlMask(), command.getRowPosition());
		return true;
	}
	
	protected void selectRows(int columnPosition, List<Integer> rowPositions, boolean withShiftMask, boolean withControlMask, int rowPositionToMoveIntoViewport) {
		Set<Range> changedRowRanges = new HashSet<Range>();
		
		if( rowPositions.size() > 0 ) {
			changedRowRanges.addAll(internalSelectRow(columnPosition, rowPositions.get(0), rowPositions.size(), withShiftMask, withControlMask));
		}

		Set<Integer> changedRows = new HashSet<Integer>();
		for (Range range : changedRowRanges) {
			for (int i = range.start; i < range.end; i++) {
				changedRows.add(Integer.valueOf(i));
			}
		}
		selectionLayer.fireLayerEvent(new RowSelectionEvent(selectionLayer, changedRows, rowPositionToMoveIntoViewport));
	}
	
	private Set<Range> internalSelectRow(int columnPosition, int rowPosition, int rowCount, boolean withShiftMask, boolean withControlMask) {
		Set<Range> changedRowRanges = new HashSet<Range>();
		
		if (noShiftOrControl(withShiftMask, withControlMask)) {
			changedRowRanges.addAll(selectionLayer.getSelectedRows());
			selectionLayer.clear();
			selectionLayer.selectCell(0, rowPosition, withShiftMask, withControlMask);
			selectionLayer.selectRegion(0, rowPosition, selectionLayer.getColumnCount(), rowCount);
			selectionLayer.moveSelectionAnchor(columnPosition, rowPosition);
			changedRowRanges.add(new Range(rowPosition, rowPosition + rowCount));
		} else if (isControlOnly(withShiftMask, withControlMask)) {
			changedRowRanges.add(selectRowWithCtrlKey(columnPosition, rowPosition, rowCount));
		}

		selectionLayer.lastSelectedCell.columnPosition = selectionLayer.getColumnCount() - 1;
		selectionLayer.lastSelectedCell.rowPosition = rowPosition;
		
		return changedRowRanges;
	}
	
	private Range selectRowWithCtrlKey(int columnPosition, int rowPosition, int rowCount) {
		Rectangle selectedRowRectangle = new Rectangle(0, rowPosition, selectionLayer.getColumnCount(), rowCount);

		if (selectionLayer.isRowFullySelected(rowPosition)) {
			selectionLayer.clearSelection(selectedRowRectangle);
			if (selectionLayer.lastSelectedRegion != null && selectionLayer.lastSelectedRegion.equals(selectedRowRectangle)) {
				selectionLayer.lastSelectedRegion = null;
			}
		} else {
			// Preserve last selected region
			if (selectionLayer.lastSelectedRegion != null) {
				selectionLayer.selectionModel.addSelection(
						new Rectangle(selectionLayer.lastSelectedRegion.x,
								selectionLayer.lastSelectedRegion.y,
								selectionLayer.lastSelectedRegion.width,
								selectionLayer.lastSelectedRegion.height));
			}
			selectionLayer.selectRegion(0, rowPosition, selectionLayer.getColumnCount(), rowCount);
			selectionLayer.moveSelectionAnchor(columnPosition, rowPosition);
		}
		
		return new Range(rowPosition, rowPosition + 1);
	}
}
