package net.sourceforge.nattable.tree.config;

import net.sourceforge.nattable.config.CellConfigAttributes;
import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.config.IConfiguration;
import net.sourceforge.nattable.layer.ILayer;
import net.sourceforge.nattable.style.CellStyleAttributes;
import net.sourceforge.nattable.style.DisplayMode;
import net.sourceforge.nattable.style.HorizontalAlignmentEnum;
import net.sourceforge.nattable.style.Style;
import net.sourceforge.nattable.tree.ITreeRowModel;
import net.sourceforge.nattable.tree.TreeLayer;
import net.sourceforge.nattable.tree.command.TreeExpandCollapseAction;
import net.sourceforge.nattable.ui.binding.UiBindingRegistry;
import net.sourceforge.nattable.ui.matcher.MouseEventMatcher;
import net.sourceforge.nattable.util.GUIHelper;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

public class DefaultTreeLayerConfiguration implements IConfiguration {

	private final ITreeRowModel<?> treeRowModel;
	private final int TREE_INDENT = 10;
	
	private final Image plusImage;
	private final Image minusImage;
	private final Image leafImage;
	
	public DefaultTreeLayerConfiguration(ITreeRowModel<?> treeRowModel) {
		this.treeRowModel = treeRowModel;
		
		// TODO: Swap ResourceManager for GUIHelper once submitted to NatTable
		this.plusImage = GUIHelper.getImage("plus");  //$NON-NLS-1$
		this.minusImage = GUIHelper.getImage("minus");  //$NON-NLS-1$
		this.leafImage =  GUIHelper.getImage(getTransparentImage(this.plusImage.getBounds())); 
	}
	
	public ImageData getTransparentImage(Rectangle bounds) {
		int depth = 2;
		RGB transparentRGB = new RGB(255, 255, 255);
		final ImageData imageData = new ImageData(bounds.width, bounds.height, depth, new PaletteData(new RGB[] {transparentRGB}));
		imageData.transparentPixel = imageData.palette.getPixel(transparentRGB);
		return imageData;
	}
	
	public void configureLayer(ILayer layer) {
		// No op
	}

	public void configureRegistry(IConfigRegistry configRegistry) {
		configRegistry.registerConfigAttribute(
				CellConfigAttributes.CELL_PAINTER,
				new DefaultTreeCellPainter(this.treeRowModel, this.TREE_INDENT, this.plusImage, this.minusImage, this.leafImage),
				DisplayMode.NORMAL,
				TreeLayer.TREE_COLUMN_CELL
		);
		configRegistry.registerConfigAttribute(
				CellConfigAttributes.CELL_STYLE,
				new Style() {{
					setAttributeValue(CellStyleAttributes.HORIZONTAL_ALIGNMENT,  HorizontalAlignmentEnum.LEFT);
				}},
				DisplayMode.NORMAL,
				TreeLayer.TREE_COLUMN_CELL
		);
	}

	public void configureUiBindings(UiBindingRegistry uiBindingRegistry) {
		uiBindingRegistry.registerFirstDoubleClickBinding(
				MouseEventMatcher.bodyLeftClick(SWT.NONE), 
				new TreeExpandCollapseAction()
		);
		
		uiBindingRegistry.registerFirstSingleClickBinding(
				new DefaultTreeMouseEventMatcher(this.treeRowModel, this.TREE_INDENT, this.plusImage.getBounds(), this.minusImage.getBounds()), 
				new TreeExpandCollapseAction()
		);
	}

}
